"""
This module contains o!rdr API Error Codes.
"""
from __future__ import annotations

from enum import IntEnum
from enum import unique

__all__ = ("ErrorCode",)


@unique
class ErrorCode(IntEnum):
    NO_ERROR = 0
    EMERGENCY_STOP = 1
    BAD_REPLAY_FILE = 2
    REPLAY_DOWNLOAD_FAILED = 3
    NO_BEATMAP_SERVER = 4
    REPLAY_FILE_CORRUPTED = 5
    INVALID_GAMEMODE = 6
    NO_REPLAY_INPUT_DATA = 7
    NO_BEATMAP_DATA = 8
    NO_BEATMAP_AUDIO = 9
    OSU_API_ERROR = 10
    REPLAY_AUTO_MODE = 11
    REPLAY_INVALID_USERNAME = 12
    BEATMAP_TOO_LONG = 13
    PLAYER_BANNED = 14
    UNKNOWN_BEATMAP = 15
    IP_BANNED = 16
    USERNAME_BANNED = 17
    UNKNOWN_RENDER_ERROR = 18
    RENDER_DOWNLOAD_FAILED = 19
    OLD_BEATMAP_DATA = 20
    REPLAY_PARSE_FAILED = 21
    FAILED_GENERATING_VIDEO = 22
    FAILED_PREPARING_RENDER = 23
    NO_BEATMAP_NAME = 24
    PARTIAL_REPLAY_INPUT_DATA = 25
    REPLAY_INVALID_MODS = 26
    RENDER_GENERAL_ERROR = 27
    RENDER_REPLAY_DOWNLOAD_FAILED = 28
    REPLAY_ALREADY_RENDERED = 29
    STAR_RATING_TOO_HIGH = 30
    MAPPER_BANNED = 31
    BEATMAPSET_BANNED = 32
    REPLAY_RECENT_FAIL = 33



"""
This module contains models for renders.
"""
from __future__ import annotations

from datetime import datetime
from enum import Enum

from pydantic import Field

from .base import BaseModel

__all__ = (
    "RenderResolution",
    "RenderOptions",
    "Render",
    "RendersResponse",
    "RenderCreateResponse",
)


class RenderResolution(Enum):
    SD_480 = "720x480"
    SD_960 = "960x540"
    HD_720 = "1280x720"
    HD_1080 = "1920x1080"


class RenderOptions(BaseModel):
    resolution: RenderResolution = Field(default=RenderResolution.HD_720)
    global_volume: int = Field(alias="globalVolume", default=50)
    music_volume: int = Field(alias="musicVolume", default=50)
    hitsound_volume: int = Field(alias="hitsoundVolume", default=50)
    show_hit_error_meter: bool = Field(alias="showHitErrorMeter", default=True)
    show_unstable_rate: bool = Field(alias="showUnstableRate", default=True)
    show_score: bool = Field(alias="showScore", default=True)
    show_hp_bar: bool = Field(alias="showHPBar", default=True)
    show_combo_counter: bool = Field(alias="showComboCounter", default=True)
    show_pp_counter: bool = Field(alias="showPPCounter", default=True)
    show_key_overlay: bool = Field(alias="showKeyOverlay", default=True)
    show_scoreboard: bool = Field(alias="showScoreboard", default=True)
    show_borders: bool = Field(alias="showBorders", default=True)
    show_mods: bool = Field(alias="showMods", default=True)
    show_result_screen: bool = Field(alias="showResultScreen", default=True)
    use_skin_cursor: bool = Field(alias="useSkinCursor", default=True)
    use_skin_hitsounds: bool = Field(alias="useSkinHitsounds", default=True)
    use_beatmap_colors: bool = Field(alias="useBeatmapColors", default=True)
    cursor_scale_to_cs: bool = Field(alias="cursorScaleToCS", default=False)
    cursor_rainbow: bool = Field(alias="cursorRainbow", default=False)
    cursor_trail_glow: bool = Field(alias="cursorTrailGlow", default=False)
    draw_follow_points: bool = Field(alias="drawFollowPoints", default=True)
    draw_combo_numbers: bool = Field(alias="drawComboNumbers", default=True)
    cursor_size: float = Field(alias="cursorSize", default=1.0)
    cursor_trail: bool = Field(alias="cursorTrail", default=True)
    beat_scaling: bool = Field(alias="scaleToTheBeat", default=False)
    slider_merge: bool = Field(alias="sliderMerge", default=False)
    objects_rainbow: bool = Field(alias="objectsRainbow", default=False)
    flash_objects: bool = Field(alias="objectsFlashToTheBeat", default=False)
    use_slider_hitcircle_color: bool = Field(alias="useHitCircleColor", default=True)
    seizure_warning: bool = Field(alias="seizureWarning", default=False)
    load_storyboard: bool = Field(alias="loadStoryboard", default=True)
    load_video: bool = Field(alias="loadVideo", default=True)
    intro_bg_dim: int = Field(alias="introBGDim", default=0)
    ingame_bg_dim: int = Field(alias="inGameBGDim", default=75)
    break_bg_dim: int = Field(alias="breakBGDim", default=30)
    bg_parallax: bool = Field(alias="BGParallax", default=False)
    show_danser_logo: bool = Field(alias="showDanserLogo", default=True)
    skip_intro: bool = Field(alias="skip", default=True)
    cursor_ripples: bool = Field(alias="cursorRipples", default=False)
    slider_snaking_in: bool = Field(alias="sliderSnakingIn", default=True)
    slider_snaking_out: bool = Field(alias="sliderSnakingOut", default=True)
    show_hit_counter: bool = Field(alias="showHitCounter", default=False)
    show_avatars_on_scoreboard: bool = Field(
        alias="showAvatarsOnScoreboard",
        default=False,
    )
    show_aim_error_meter: bool = Field(alias="showAimErrorMeter", default=False)
    play_nightcore_samples: bool = Field(alias="playNightcoreSamples", default=True)


class Render(RenderOptions):
    id: int = Field(alias="renderID")
    date: datetime
    username: str
    progress: str
    renderer: str
    description: str
    title: str
    readable_date: str = Field(alias="readableDate")
    is_bot: bool = Field(alias="isBot")
    is_verified: bool = Field(alias="isVerified")
    replay_file_path: str = Field(alias="replayFilePath")
    video_url: str = Field(alias="videoUrl")
    map_link: str = Field(alias="mapLink")
    map_title: str = Field(alias="mapTitle")
    replay_difficulty: str = Field(alias="replayDifficulty")
    replay_username: str = Field(alias="replayUsername")
    map_id: int = Field(alias="mapID")
    need_to_redownload: bool = Field(alias="needToRedownload")
    skin: str
    has_cursor_middle: bool = Field(alias="hasCursorMiddle")
    motion_blur: bool = Field(alias="motionBlur960fps")
    render_start_time: datetime = Field(alias="renderStartTime")
    render_end_time: datetime = Field(alias="renderEndTime")
    upload_end_time: datetime = Field(alias="uploadEndTime")
    render_total_time: int = Field(alias="renderTotalTime")
    upload_total_time: int = Field(alias="uploadTotalTime")
    map_length: int = Field(alias="mapLength")
    replay_mods: str = Field(alias="replayMods")
    removed: bool


class RendersResponse(BaseModel):
    renders: list[Render]
    max_renders: int = Field(alias="maxRenders")


class RenderCreateResponse(BaseModel):
    message: str
    render_id: int = Field(alias="renderID")



"""This module contains models for the o!rdr API."""
from __future__ import annotations

from .base import *
from .errorcode import *
from .events import *
from .render import *
from .renderserver import *
from .skin import *



"""
This module contains models for render servers.
"""
from __future__ import annotations

from datetime import datetime

from pydantic import Field

from .base import BaseModel

__all__ = (
    "RenderServerOptions",
    "RenderServer",
)


class RenderServerOptions(BaseModel):
    text_color: str = Field(alias="textColor")
    background_type: str = Field(alias="backgroundType")


class RenderServer(BaseModel):
    enabled: bool
    last_seen: datetime = Field(alias="lastSeen")
    name: str
    priority: float
    old_score: float = Field(alias="oldScore")
    avg_fps: int = Field(alias="avgFPS")
    power: str
    status: str
    total_rendered: int = Field(alias="totalRendered")
    rendering_type: str = Field(alias="renderingType")
    cpu: str
    gpu: str
    motion_blur_capable: bool = Field(alias="motionBlurCapable")
    using_osu_api: bool = Field(alias="usingOsuApi")
    uhd_capable: bool = Field(alias="uhdCapable")
    avg_render_time: float = Field(alias="avgRenderTime")
    avg_upload_time: float = Field(alias="avgUploadTime")
    total_avg_time: float = Field(alias="totalAvgTime")
    total_uploaded_videos_size: int = Field(alias="totalUploadedVideosSize")
    owner_user_id: int = Field(alias="ownerUserId")
    owner_username: str = Field(alias="ownerUsername")
    customization: RenderServerOptions



"""
This module contains custom exception types.
"""
from __future__ import annotations

from .models import ErrorCode

__all__ = ("APIException",)


class APIException(Exception):
    """API Exception Class

    :param status: status code from the API
    :type status: int
    :param message: error message returned
    :type message: str
    :param error_code: error code returned
    :type error_code: aiordr.models.errorcode.ErrorCode
    """

    def __init__(self, status: int, message: str, error_code: ErrorCode) -> None:
        super().__init__(message)
        self.status = status
        self.error_code = error_code

    @property
    def message(self) -> str:
        """Error message returned by the API

        :return: Error message
        :rtype: str
        """
        return self.args[0]



"""
This module contains models for websocket events.
"""
from __future__ import annotations

from pydantic import Field

from .base import BaseModel
from .errorcode import ErrorCode

__all__ = (
    "RenderBaseEvent",
    "RenderAddEvent",
    "RenderProgressEvent",
    "RenderFinishEvent",
    "RenderFailEvent",
)


class RenderBaseEvent(BaseModel):
    render_id: int = Field(alias="renderID")


class RenderAddEvent(RenderBaseEvent):
    pass


class RenderProgressEvent(RenderBaseEvent):
    username: str
    """Username of the user who requested the render."""
    progress: str
    """Status text of the render process."""
    renderer: str
    """Name of the render server."""
    description: str
    """Video description."""


class RenderFinishEvent(RenderBaseEvent):
    video_url: str = Field(alias="videoUrl")


class RenderFailEvent(RenderBaseEvent):
    error_message: str = Field(alias="errorMessage")
    error_code: ErrorCode = Field(alias="errorCode")



"""This module contains the client for interfacing with the o!rdr API."""
from __future__ import annotations

import functools
from typing import Literal
from typing import TYPE_CHECKING
from warnings import warn

import aiohttp
import orjson
from aiolimiter import AsyncLimiter
from socketio import AsyncClient as sio_async  # type: ignore

from .exceptions import APIException
from .helpers import add_param
from .helpers import from_list
from .models import ErrorCode
from .models import RenderAddEvent
from .models import RenderCreateResponse
from .models import RenderFailEvent
from .models import RenderFinishEvent
from .models import RenderOptions
from .models import RenderProgressEvent
from .models import RenderServer
from .models import RendersResponse
from .models import SkinCompact
from .models import SkinsResponse

if TYPE_CHECKING:
    from types import TracebackType
    from typing import Any
    from typing import Type
    from typing import Union
    from typing import Optional
    from typing import Callable


__all__ = ("ordrClient", "DeveloperModes")

ClientRequestType = Literal["GET", "POST", "DELETE", "PUT", "PATCH"]


def get_content_type(content_type: str) -> str:
    """Returns the content type."""
    return content_type.split(";")[0]


DeveloperModes = Literal["devmode_success", "devmode_fail", "devmode_wsfail"]


class ordrClient:
    def __init__(self, **kwargs: Any) -> None:
        r"""o!rdr API client.
        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *developer_mode* (``DeveloperModes``) --
                Optional, defaults to None
            * *verification_key* (``str``) --
                Optional, defaults to None. If not provided, rate limits will be forced to 1 request per 5 minutes
            * *limiter* (``tuple[int, int]``) --
                Optional, rate limit, defaults to (1, 300) (1 requests per 5 minutes)
        """
        self._developer_mode: Optional[str] = kwargs.pop("developer_mode", None)
        self._verification_key: Optional[str] = kwargs.pop("verification_key", None)

        if self._developer_mode:
            if self._verification_key:
                warn(
                    "You are running in developer mode. This means that your requests will be simulated and your verification key will not be used.",
                )
            self._verification_key = self._developer_mode

        self._session: Optional[aiohttp.ClientSession] = None
        self._base_url: str = "https://apis.issou.best"
        self._websocket_url: str = "https://ordr-ws.issou.best"

        max_rate, time_period = kwargs.pop("limiter", (1, 300))
        if (max_rate / time_period) > 1:
            warn(
                "You are running at an insanely high rate limit. Doing so may result in your account being banned.",
            )

        if not self._verification_key:
            max_rate = 1
            time_period = 300

        self._limiter: AsyncLimiter = AsyncLimiter(
            max_rate=max_rate,
            time_period=time_period,
        )

        self.socket = sio_async()

    def on_render_added(self, func: Callable) -> Callable:
        r"""Returns a callable that is called when a render is added, to be used as:
        @client.on_render_added()
        async def render_added(event: RenderAddEvent):
        """

        @functools.wraps(func)
        async def wrapper(data: dict) -> Any:
            return await func(RenderAddEvent.parse_obj(data))

        self.socket.on("render_added_json", wrapper)
        return wrapper

    def on_render_progress(self, func: Callable) -> Callable:
        r"""Returns a callable that is called when a render is updated, to be used as:
        @client.on_render_progress()
        async def render_progress(event: RenderProgressEvent):
        """

        @functools.wraps(func)
        async def wrapper(data: dict) -> Any:
            return await func(RenderProgressEvent.parse_obj(data))

        self.socket.on("render_progress_json", wrapper)
        return wrapper

    def on_render_fail(self, func: Callable) -> Callable:
        r"""Returns a callable that is called when a render fails, to be used as:
        @client.on_render_fail()
        async def render_fail(event: RenderFailEvent):
        """

        @functools.wraps(func)
        async def wrapper(data: dict) -> Any:
            return await func(RenderFailEvent.parse_obj(data))

        self.socket.on("render_fail_json", wrapper)
        return wrapper

    def on_render_finish(self, func: Callable) -> Callable:
        r"""Returns a callable that is called when a render finishes, to be used as:
        @client.on_render_finish()
        async def render_finish(event: RenderFinishEvent):
        """

        @functools.wraps(func)
        async def wrapper(data: dict) -> Any:
            return await func(RenderFinishEvent.parse_obj(data))

        self.socket.on("render_done_json", wrapper)
        return wrapper

    async def __aenter__(self) -> ordrClient:
        self._session = aiohttp.ClientSession()
        await self.connect()
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> None:
        await self.close()

    async def _request(
        self, request_type: ClientRequestType, *args: Any, **kwargs: Any
    ) -> Any:
        if not self.socket.connected:
            await self.connect()
        if self._session is None:
            self._session = aiohttp.ClientSession()

        req: dict[str, Callable] = {
            "GET": self._session.get,
            "POST": self._session.post,
            "DELETE": self._session.delete,
            "PUT": self._session.put,
            "PATCH": self._session.patch,
        }

        async with self._limiter:
            async with req[request_type](*args, **kwargs) as resp:
                body = await resp.read()
                content_type = get_content_type(resp.headers.get("content-type", ""))
                if resp.status not in (200, 201):
                    json = orjson.loads(body)
                    error_code = json.get("errorCode", 0)
                    raise APIException(
                        resp.status,
                        json.get("message", ""),
                        ErrorCode(error_code),
                    )
                if content_type == "application/json":
                    return orjson.loads(body)
                if content_type == "text/html":
                    return body.decode("utf-8")
                raise APIException(415, "Unhandled Content Type", ErrorCode(0))

    async def get_custom_skin(self, skin_id: int) -> SkinCompact:
        r"""Get custom skin information.

        :param skin_id: Skin ID
        :type skin_id: ``int``
        :raises: ``aiordr.exceptions.APIException``: Contains status code, error message, and error code
        :return: Skin information
        :rtype: ``aiordr.models.skin.SkinCompact``
        """
        params = {"id": skin_id}
        json = await self._request(
            "GET",
            f"{self._base_url}/ordr/skins/custom",
            params=params,
        )
        return SkinCompact.parse_obj(json)

    async def get_skins(
        self, page: int = 1, page_size: int = 5, **kwargs: Any
    ) -> SkinsResponse:
        r"""Get custom skins.

        :param page: Page number
        :type page: ``int``
        :param page_size: Page size
        :type page_size: ``int``
        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *search* (``str``) --
                Optional, search query

        :raises: ``aiordr.exceptions.APIException``: Contains status code, error message, and error code
        :return: Skins
        :rtype: ``aiordr.models.skins.SkinsResponse``
        """
        params = {
            "page": page,
            "pageSize": page_size,
        }
        add_param(params, kwargs, "search")
        json = await self._request(
            "GET",
            f"{self._base_url}/ordr/skins",
            params=params,
        )
        return SkinsResponse.parse_obj(json)

    async def get_render_list(
        self, page: int = 1, page_size: int = 5, **kwargs: Any
    ) -> RendersResponse:
        r"""Get render list.

        :param page: Page number
        :type page: ``int``
        :param page_size: Page size
        :type page_size: ``int``
        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *ordr_username* (``str``) --
                Optional, username of the user who ordered the render
            * *replay_username* (``str``) --
                Optional, username of the user from the replay
            * *render_id* (``int``) --
                Optional, ID of the render
            * *no_bots* (``bool``) --
                Optional, whether to exclude bot renders
            * *link* (``str``) --
                Optional, the path of a shortlink (e.g. pov8n for https://link.issou.best/pov8n)
            * *beatmapset_id* (``int``) --
                Optional, ID of the beatmapset

        :raises: ``aiordr.exceptions.APIException``: Contains status code, error message, and error code
        :return: Renders
        :rtype: ``aiordr.models.renders.RendersResponse``
        """
        params = {
            "page": page,
            "pageSize": page_size,
        }
        add_param(params, kwargs, "ordr_username", "ordrUsername")
        add_param(params, kwargs, "replay_username", "replayUsername")
        add_param(params, kwargs, "render_id", "renderID")
        add_param(params, kwargs, "no_bots", "nobots")
        add_param(params, kwargs, "link")
        add_param(params, kwargs, "beatmapset_id", "beatmapsetid")
        json = await self._request(
            "GET",
            f"{self._base_url}/ordr/renders",
        )
        return RendersResponse.parse_obj(json)

    async def get_server_list(self) -> list[RenderServer]:
        r"""Get the list of available servers.


        :raises: ``aiordr.exceptions.APIException``: Contains status code, error message, and error code
        :return: List of servers
        :rtype: ``list[aiordr.models.server.RenderServer]``
        """
        json = await self._request(
            "GET",
            f"{self._base_url}/servers",
        )
        return from_list(RenderServer.parse_obj, json.get("servers", []))

    async def get_server_online_count(self) -> int:
        r"""Get the number of online servers.


        :raises: ``aiordr.exceptions.APIException``: Contains status code, error message, and error code
        :return: Number of online servers
        :rtype: ``int``
        """
        data = await self._request(
            "GET",
            f"{self._base_url}/servers/onlinecount",
        )
        try:
            return int(data)
        except ValueError:
            return 0

    async def create_render(
        self, username: str, skin: Union[str, int], **kwargs: Any
    ) -> RenderCreateResponse:
        r"""Create a render.

        :param username: Username of the user who ordered the render
        :type username: ``str``
        :param skin: Skin ID or name
        :type skin: ``Union[str, int]``
        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *replay_file* (``typing.IO``) --
                Optional, replay file
            * *replay_url* (``str``) --
                Optional, replay URL, used if replay_file is not provided
            * *render_options* (``aiordr.models.render.RenderOptions``) --
                Optional, render options
            * *custom_skin* (``bool``) --
                Optional, whether the provided skin is a custom skin ID (default: false)

        :raises: ``aiordr.exceptions.APIException``: Contains status code, error message, and error code
        :raises: ``TypeError``: If render_options is not a RenderOptions object
        :return: Render create response
        :rtype: ``aiordr.models.render.RenderCreateResponse``
        """

        if "replay_file" not in kwargs and "replay_url" not in kwargs:
            raise ValueError("Either replay_file or replay_url must be provided")

        data = {
            "username": username,
            "skin": skin,
        }
        if self._verification_key:
            data["verificationKey"] = self._verification_key

        if "render_options" not in kwargs:
            kwargs["render_options"] = RenderOptions()
        options: RenderOptions = kwargs["render_options"]
        if not isinstance(options, RenderOptions):
            raise TypeError("render_options must be a RenderOptions object")

        data.update(options.dict(exclude_defaults=True, by_alias=True))
        data["resolution"] = options.resolution.value

        add_param(data, kwargs, "replay_url", "replayURL")
        add_param(data, kwargs, "custom_skin", "customSkin")

        form_data = aiohttp.FormData()
        for key, value in data.items():
            if isinstance(value, bool):
                value = str(value).lower()
            form_data.add_field(key, value)

        if "replay_file" in kwargs:
            form_data.add_field(
                "replayFile",
                kwargs["replay_file"],
                filename="replay.osr",
            )

        json = await self._request(
            "POST",
            f"{self._base_url}/ordr/renders",
            data=form_data,
        )
        return RenderCreateResponse.parse_obj(json)

    async def connect(self) -> None:
        r"""Connects to the websocket server.

        :return: None
        """
        await self.socket.connect(url=self._websocket_url)

    async def close(self) -> None:
        r"""Closes the client.

        :return: None
        """
        if self._session is not None:
            await self._session.close()
        await self.socket.disconnect()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aiordr', 'aiordr.models']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.3,<4.0.0',
 'aiolimiter>=1.0.0,<2.0.0',
 'orjson>=3.8.3,<4.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'python-socketio[asyncio-client]>=5.7.2,<6.0.0']

extras_require = \
{'docs': ['toml>=0.10.2,<0.11.0',
          'sphinx>=6.0.0,<7.0.0',
          'sphinx-rtd-theme>=1.1.1,<2.0.0'],
 'test': ['pytest>=7.2.0,<8.0.0',
          'pytest-asyncio>=0.20.2,<0.21.0',
          'pytest-mock>=3.10.0,<4.0.0',
          'toml>=0.10.2,<0.11.0',
          'types-toml>=0.10.8.1,<0.11.0.0']}

setup_kwargs = {
    'name': 'aiordr',
    'version': '0.0.4',
    'description': 'Simple and fast library for interacting with the o!rdr API.',
    'long_description': 'aiordr\n======\n\n|Python| |pypi| |pre-commit.ci status| |rtd| |pytest| |mypy| |codacy|\n\nSimple and fast asynchronous library for the o!rdr API.\n\n\nFeatures\n--------\n\n- Support for modern async syntax (async with)\n- Event decorators\n- Rate limit handling\n- Easy to use\n\n\nInstalling\n----------\n\n**Python 3.9 or higher is required**\n\nTo install the library, simply run the following commands\n\n.. code:: sh\n\n    # Linux/macOS\n    python3 -m pip install -U aiordr\n\n    # Windows\n    py -3 -m pip install -U aiordr\n\n\nTo install the development version, do the following:\n\n.. code:: sh\n\n    $ git clone https://github.com/NiceAesth/aiordr\n    $ cd aiordr\n    $ python3 -m pip install -U .\n\n\nAPI Example\n-----------\n\n.. code:: py\n\n    import aiordr\n    import asyncio\n\n\n    async def main():\n        client = aiordr.ordrClient(verification_key="verylongstring")\n\n        await client.create_render(\n            "username",\n            "YUGEN",\n            replay_url="https://url.to.replay",\n        )\n\n        @client.on_render_added\n        async def on_render_added(event: aiordr.models.RenderAddEvent) -> None:\n            print(event)\n\n        @client.on_render_progress\n        async def on_render_progress(event: aiordr.models.RenderProgressEvent) -> None:\n            print(event)\n\n        @client.on_render_fail\n        async def on_render_fail(event: aiordr.models.RenderFailEvent) -> None:\n            print(event)\n\n        @client.on_render_finish\n        async def on_render_finish(event: aiordr.models.RenderFinishEvent) -> None:\n            print(event)\n\n\n    if __name__ == "__main__":\n        asyncio.run(main())\n\n\nContributing\n------------\n\nPlease read the `CONTRIBUTING.rst <.github/CONTRIBUTING.rst>`__ to learn how to contribute to aiordr!\n\n\nAcknowledgments\n---------------\n\n-  `discord.py <https://github.com/Rapptz/discord.py>`__\n   for README formatting\n-  `aiosu <https://github.com/NiceAesth/aiosu>`__\n   sister library for the osu! API\n\n\n.. |Python| image:: https://img.shields.io/pypi/pyversions/aiordr.svg\n    :target: https://pypi.python.org/pypi/aiordr\n    :alt: Python version info\n.. |pypi| image:: https://img.shields.io/pypi/v/aiordr.svg\n    :target: https://pypi.python.org/pypi/aiordr\n    :alt: PyPI version info\n.. |pre-commit.ci status| image:: https://results.pre-commit.ci/badge/github/NiceAesth/aiordr/master.svg\n    :target: https://results.pre-commit.ci/latest/github/NiceAesth/aiordr/master\n    :alt: pre-commit.ci status\n.. |pytest| image:: https://github.com/NiceAesth/aiordr/actions/workflows/pytest.yml/badge.svg\n    :target: https://github.com/NiceAesth/aiordr/actions/workflows/pytest.yml\n    :alt: pytest Status\n.. |mypy| image:: https://github.com/NiceAesth/aiordr/actions/workflows/mypy.yml/badge.svg\n    :target: https://github.com/NiceAesth/aiordr/actions/workflows/mypy.yml\n    :alt: mypy Status\n.. |rtd| image:: https://readthedocs.org/projects/aiordr/badge/?version=latest\n    :target: https://aiordr.readthedocs.io/en/latest/?badge=latest\n    :alt: Documentation Status\n.. |codacy| image:: https://app.codacy.com/project/badge/Grade/4778d5ee1dc84469ad6a43a6f961c0eb\n    :target: https://www.codacy.com/gh/NiceAesth/aiordr/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=NiceAesth/aiordr&amp;utm_campaign=Badge_Grade\n    :alt: Codacy Status\n',
    'author': 'Nice Aesthetics',
    'author_email': 'nice@aesth.dev',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/NiceAesth/aiordr',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""
This module contains models for skins.
"""
from __future__ import annotations

from pydantic import Field

from .base import BaseModel

__all__ = (
    "Skin",
    "SkinCompact",
    "SkinsResponse",
)


class Skin(BaseModel):
    id: int
    name: str = Field(alias="skin")
    presentation_name: str = Field(alias="presentationName")
    url: str
    high_res_preview_url: str = Field(alias="highResPreview")
    low_res_preview_url: str = Field(alias="lowResPreview")
    grid_preview_url: str = Field(alias="gridPreview")
    has_cursor_middle: bool = Field(alias="hasCursorMiddle")
    modified: bool
    author: str
    version: str
    times_used: int = Field(alias="timesUsed")


class SkinCompact(BaseModel):
    found: bool
    removed: bool
    message: str
    skin_name: str = Field(alias="skinName")
    skin_author: str = Field(alias="skinAuthor")
    download_link: str = Field(alias="downloadLink")


class SkinsResponse(BaseModel):
    skins: list[Skin]
    message: str
    max_skins: int = Field(alias="maxSkins")
